<?php
/*
* $Id: st_name_based_urls.php
* $Loc: /includes/modules/name_based_urls/
*
* Name: NameBasedURLsDemo
* Version: 1.4
* Release Date: 07/21/2024
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under Commercial License
*
* Comments:  Author: [Rainer Schmied @raiwa]
*  Author URI: [www.phoenixcartaddons.com]
*  License: Commercial License Rainer Schmied
*  License URI: https://www.phoenixcartaddons.com/license.php
* 
*  Copyright (c) 2024 Rainer Schmied
*  
* and guidance by
* Matt @ecartz
* https://phoenixcart.org
* 
*  Redistribution and use in source and binary forms, with or without
*  modification, are not permitted without specific prior written permission
*  from Rainer Schmied
* 
* based on:
* Ultimate SEO URLs Contribution - osCommerce MS-2.2
* http://www.oscommerce-freelancers.com/ osCommerce-Freelancers
* Copyright 2005, Bobby Easland
* Bobby Easland
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
*  SOFTWARE.
*  
* 
*/

  class st_name_based_urls extends abstract_executable_module {

    const CONFIG_KEY_BASE = 'MODULE_NB_URLS_';

    public function __construct() {
      parent::__construct(__FILE__);

      $this->description .= '<p>by @raiwa <u><a target="_blank" href="http://www.oscaddons.com">www.oscaddons.com</a></u></p>';
      $this->description .= '<p>Get <u><a target="_blank" href="https://www.oscaddons.com/en/commercial-add-ons/name-based-urls-pro-p-89.html">Name-based URLs Pro</a></u></p>';
    }

    function execute() {

    }

    protected function get_parameters() {
      $config_parameters = [
        static::CONFIG_KEY_BASE . 'VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '1.0.0 Demo',
          'desc' => 'Version info. It is read only',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'STATUS' => [
          'title' => 'Enable Name Based URLs Module',
          'value' => 'True',
          'desc' => 'Do you want to add the Name Based URLs to your shop?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        static::CONFIG_KEY_BASE . 'ADD_CID_TO_PRODUCT_URLS' => [
          'title' => 'Add cPath to product URLs? [Pro Version]',
          'value' => 'True',
          'desc' => 'This setting will append the cPath to the end of product URLs (i.e. - some-product-p-1.html?cPath=xx).<br>It is used to show the correct Category Path for duplicated products in different categories.',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'ADD_CPATH_TO_PRODUCT_URLS' => [
          'title' => 'Add category parent to product URLs? [Pro Version]',
          'value' => 'False',
          'desc' => 'This setting will append the category parent(s) name to the product URLs (i.e. - parent-some-product-p-1.html).',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'ADD_CAT_PARENT' => [
          'title' => 'Add category parent to begining of category URLs? [Pro Version]',
          'value' => 'False',
          'desc' => 'This setting will add the category parent(s) name to the beginning of the category URLs (i.e. - parent-category-c-1.html).',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'PATH_SEPARATOR' => [
          'title' => 'Path separator [Pro Version]',
          'value' => '-',
          'desc' => 'This setting selects the path separator to use if the category parent is added to catgories or products URLs.',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'FILTER_SHORT_WORDS' => [
          'title' => 'Filter Short Words',
          'value' => '3',
          'desc' => 'This setting will filter words less than or equal to the value from the URL.',
        ],
        static::CONFIG_KEY_BASE . 'USE_SEO_REDIRECT' => [
          'title' => 'Enable automatic redirects? [Pro Version]',
          'value' => 'False',
          'desc' => 'This will activate the automatic redirect code and send 301 headers for old to new URLs.',
          'set_func' => 'st_name_based_urls::readonly(',
        ],
        static::CONFIG_KEY_BASE . 'USE_SEO_TITLES' => [
          'title' => 'Enable use SEO Titles as name?',
          'value' => 'False',
          'desc' => 'This will cause the title set in Category/Products/Manufacturer Title for SEO to be used instead of the categories or products name.',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        static::CONFIG_KEY_BASE . 'REMOVE_ALL_SPEC_CHARS' => [
          'title' => 'Remove all non-alphanumeric characters?',
          'value' => 'False',
          'desc' => 'This will remove all non-letters and non-numbers.  This should be handy to remove all special characters with 1 setting.',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
      ];
      $lng = new language;
      foreach($lng->catalog_languages as $id => $value) {
      	$key = strtoupper($value['directory']);
        switch ($key) {
        	case 'ESPANOL':
            $config_parameters[static::CONFIG_KEY_BASE . 'CHAR_CONVERT_SET_' . $key] = [
                'title' => 'Conversiones de caracteres especiales para ' . $key . ' [Pro Version]',
                'value' => '',
                'desc' => 'Esta configuración convertirá los caracteres.<br><br>El formato <b>DEBE</b> estar en la forma: <b>char=>conv,char2=>conv2</b>',
                'set_func' => 'st_name_based_urls::readonly(',
            ];
       		break;
        	case 'GERMAN':
            $config_parameters[static::CONFIG_KEY_BASE . 'CHAR_CONVERT_SET_' . $key] = [
                'title' => 'Sonderzeichenkonvertierungen für ' . $key . ' [Pro Version]',
                'value' => '',
                'desc' => 'Diese Einstellung konvertiert Zeichen.<br><br>Es <b>MUSS</b> folgendes Format haben: <b>char=>conv,char2=>conv2</b>',
                'set_func' => 'st_name_based_urls::readonly(',
            ];
            break;
        	default:
            $config_parameters[static::CONFIG_KEY_BASE . 'CHAR_CONVERT_SET_' . $key] = [
                'title' => 'Special character conversions for ' . $key . ' [Pro Version]',
                'value' => '',
                'desc' => 'This setting will convert characters.<br><br>The format <b>MUST</b> be in the form: <b>char=>conv,char2=>conv2</b>',
                'set_func' => 'st_name_based_urls::readonly(',
            ];
        }
      }

      return $config_parameters;
    }

    function install($parameter_key = null) {
      parent::install($parameter_key);

      $GLOBALS['db']->query("UPDATE hooks
                             SET hooks_method = 'name_based_urls::listen_hrefLink'
                             WHERE hooks_site = 'shop'
                             AND hooks_group = 'system'
                             AND hooks_action = 'hrefLink'
                             AND hooks_code = '_01_href_link'
                             AND hooks_class = ''
                            ");

      st_name_based_urls::htaccess_backup('master_backup_htaccess_' . date('Y-m-d'));
      st_name_based_urls::add_to_htaccess();

    }

    public function remove() {
      parent::remove();

      $GLOBALS['db']->query("UPDATE hooks
                             SET hooks_method = 'Href::hook'
                             WHERE hooks_site = 'shop'
                             AND hooks_group = 'system'
                             AND hooks_action = 'hrefLink'
                             AND hooks_code = '_01_href_link'
                             AND hooks_class = ''
                            ");

      $file = file_get_contents(DIR_FS_CATALOG . '.htaccess');
      if ( strpos($file, '# Begin Name-based URLs') !== false ) {
        $file = preg_replace('@# Begin Name-based URLs((?:.|[\r\n])+?)# End Name-based URLs@', '', $file);
        file_put_contents(DIR_FS_CATALOG . '.htaccess', $file);
         $GLOBALS['messageStack']->add_session('Name Based URLs rewrite rules have been successfully removed from your main .htaccess file.', 'success');
      }

    }

    public static function readonly($value) {
      return $value;
    }

    public static function cfg_module_show_text($value) {
      return str_replace('<br>', '', $value);
    }

    public static function cfg_module_textarea($text, $key = '') {
      $name = (($key) ? 'configuration[' . $key . ']' : 'configuration_value');
      return '<textarea id="id_' . $name  . '" name="' . $name  . '" rows="4" cols="35" nowrap>' . Text::prepare($text) . '</textarea>';
    }

    public static function add_to_htaccess() {
      
      $rewrite  = '# Begin Name-based URLs ' .  PHP_EOL .
                  'Options +FollowSymLinks' .  PHP_EOL .
                  'RewriteEngine On'  .  PHP_EOL .
                  PHP_EOL .
                  '# RewriteBase instructions'  .  PHP_EOL .
                  '# Change RewriteBase dependent on how your shop is accessed as below.'  .  PHP_EOL .
                  '# http://www.mysite.com = RewriteBase /'  .  PHP_EOL .
                  '# http://www.mysite.com/catalog/ = RewriteBase /catalog/'  .  PHP_EOL .
                  '# http://www.mysite.com/catalog/shop/ = RewriteBase /catalog/shop/ '  .  PHP_EOL .
                  PHP_EOL .
                  '# Change the following line using the instructions above '  .  PHP_EOL .
                  'RewriteBase ' . DIR_WS_CATALOG .  PHP_EOL .
                  PHP_EOL .
                  'RewriteRule ^(.*)-p-(.*).html$ product_info.php?products_id=$2&%{QUERY_STRING}' .  PHP_EOL . 
                  'RewriteRule ^(.*)-c-(.*).html$ index.php?cPath=$2&%{QUERY_STRING}' .  PHP_EOL .
                  'RewriteRule ^(.*)-m-(.*).html$ index.php?manufacturers_id=$2&%{QUERY_STRING}' .  PHP_EOL .
                  'RewriteRule ^(.*)-ip-(.*).html$ info.php?pages_id=$2&%{QUERY_STRING}' .  PHP_EOL .
                  'RewriteRule ^(.*)-pr-(.*).html$ ext/modules/content/reviews/write.php?products_id=$2&%{QUERY_STRING}' .  PHP_EOL .
                  '# End Name-based URLs';
      $rewrite_exists  = ['# Begin Name-based URLs',
                          'RewriteRule ^(.*)-p-(.*).html$ product_info.php?products_id=$2&%{QUERY_STRING}',
                          'RewriteRule ^(.*)-c-(.*).html$ index.php?cPath=$2&%{QUERY_STRING}',
                          'RewriteRule ^(.*)-m-(.*).html$ index.php?manufacturers_id=$2&%{QUERY_STRING}',
                          'RewriteRule ^(.*)-ip-(.*).html$ info.php?pages_id=$2&%{QUERY_STRING}',
                          'RewriteRule ^(.*)-pr-(.*).html$ ext/modules/content/reviews/write.php?products_id=$2&%{QUERY_STRING}',
                          '# End Name-based URLs'];
      // update the .htaccess file
      $file = file_get_contents(DIR_FS_CATALOG . '.htaccess');
      $success_message = '';
      if ( strpos($file, 'RewriteEngine On') === false && strpos($file, '# Begin Name-based URLs') === false) {
        $file .= $rewrite;
        $success_message = 'Name based rewrite rules have been successfully added to your main .htaccess file.';
        file_put_contents(DIR_FS_CATALOG . '.htaccess', $file);
      } elseif ( strpos($file, '# Begin Name-based URLs') === false) {
        $file_array = explode("\n", $file);
        for ($i=0, $n=sizeof($file_array); $i<$n; $i++) {
          if (strpos($file_array[$i], 'RewriteBase') !== false && strpos($file_array[$i], '#') === false) {
            array_splice($file_array, $i+1, 0, $rewrite_exists);
            break;
          }
        }
        $file = implode("\n", $file_array);
        $success_message = 'Name based rewrite rules have been successfully added to your main .htaccess file.<br>
                            They have been added to your existing rewrite rules.<br>
                            You may need to manually check that they have been added in the correct order.';
        file_put_contents(DIR_FS_CATALOG . '.htaccess', $file);
      }
      // check installation
      $file = file_get_contents(DIR_FS_CATALOG . '.htaccess');
      if ( strpos($file, '# Begin Name-based URLs') !== false && strpos($file, '# End Name-based URLs') !== false) {
        $GLOBALS['messageStack']->add_session($success_message, 'success');
      } else {
        $error_message = 'An error occured trying to add Name based rewrite rules to your main .htaccess file. Please check the instructions.';
        $GLOBALS['messageStack']->add_session($error_message, 'warning');
      }
    }

     // function create htaccess backup
    public static function htaccess_backup($bkup_name) {
      $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
      $backupDir = DIR_FS_CATALOG . $separator . 'htaccess_backups/';

      // create backup dir
      if(!is_dir($backupDir)) mkdir($backupDir, 0755);

      // create .htaccess protection containing allowed ip's
      $htaccessfile = $backupDir . '.htaccess';
      $htaccess = '';
      file_put_contents($htaccessfile, $htaccess);

      $htaccessfileOrig = DIR_FS_CATALOG . $separator . '.htaccess';
      $htaccessfileBkup = $backupDir . $bkup_name;

      $result = copy($htaccessfileOrig, $htaccessfileBkup);
  }

} // end class
