<?php
/*
* $Id: name_based_urls.php
* $Loc: /includes/system/override/
*
* Name: NameBasedURLsDemo
* Version: 1.4
* Release Date: 07/21/2024
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under Commercial License
*
* Comments:  Author: [Rainer Schmied @raiwa]
*  Author URI: [www.phoenixcartaddons.com]
*  License: Commercial License Rainer Schmied
*  License URI: https://www.phoenixcartaddons.com/license.php
* 
*  Copyright (c) 2024 Rainer Schmied
*  
* and guidance by
* Matt @ecartz
* https://phoenixcart.org
* 
*  Redistribution and use in source and binary forms, with or without
*  modification, are not permitted without specific prior written permission
*  from Rainer Schmied
* 
* based on:
* Ultimate SEO URLs Contribution - osCommerce MS-2.2
* http://www.oscommerce-freelancers.com/ osCommerce-Freelancers
* Copyright 2005, Bobby Easland
* Bobby Easland
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
*  SOFTWARE.
*  
* 
*/

class name_based_urls {
  /**
   * $attributes array contains all the required settings for class
  * @var array
   */
  var $attributes;
  /**
   * $base_url is the NONSSL URL for site
  * @var string
   */
  var $base_url;
  /**
   * $reg_anchors holds the anchors used by the .htaccess rewrites
  * @var array
   */
  var $reg_anchors;
  /**
   * $need_redirect determines whether the URL needs to be redirected
  * @var boolean
   */
  var $is_seopage;
  /**
   * $uri contains the $_SERVER['REQUEST_URI'] value
  * @var string
   */
  var $uri;
  /**
   * $real_uri contains the $_SERVER['SCRIPT_NAME'] . '?' . $_SERVER['QUERY_STRING'] value
  * @var string
   */
  var $real_uri;
  /**
   * $uri_parsed contains the parsed uri value array
  * @var array
   */
  var $uri_parsed;
  /**
   * $path_info contains the getenv('PATH_INFO') value
  * @var string
   */
  var $path_info;

/**
 * SEO_URL class constructor
 * @author Bobby Easland, Rainer Schmied
 * @version 1.1
 */
  function __construct() {
    global $SID;

//ojp FILENAME_LINKS
    $seo_pages =  ['index.php',
                   'product_info.php',
                  ];

//ojp USE_SEO_CACHE_LINKS
    $this->attributes =  ['PHP_VERSION' => PHP_VERSION,
                          'SESSION_STARTED' => Session::is_started(),
                          'SID' => $SID,
                          'SEO_PAGES' => $seo_pages
                          ];

    $Linker = isset($GLOBALS['Admin'])
            ? $GLOBALS['Admin']->get_catalog_linker()
            : Guarantor::ensure_global('Linker');
    $this->base_url = null !== $Linker->get_prefix()
                      ? $Linker->get_prefix()
                      : Guarantor::ensure_global('Linker')->get_prefix;
    $this->reg_anchors =  ['products_id' => '-p-',
                          ];

    if ( defined('MODULE_NB_URLS_STATUS') && MODULE_NB_URLS_STATUS == 'True' && MODULE_NB_URLS_USE_SEO_REDIRECT == 'True'){
      $this->check_redirect();
    } # end if
  } # end constructor

/**
 * Function to hook into href class and return seo url
 * @author Rainer Schmied, @ecartz
 * @version 1.0
 * @parameters array incl original link and parameters
 * @param string $parameters URL parameters
 * @param boolean $add_session_id Switch to add osCsid
 * @return Parameters array with SEO link
 */
  static function listen_hrefLink($parameters) {
    if ( defined('MODULE_NB_URLS_STATUS') && MODULE_NB_URLS_STATUS == 'True' ) {
      $name_based_urls = &Guarantor::ensure_global('name_based_urls');

      $page = str_replace($name_based_urls->base_url, '', $parameters['href']->get_page());
      if ( in_array($page, $name_based_urls->attributes['SEO_PAGES']) ) {
        $parameters['link'] = $name_based_urls->href_link($page, $parameters['href']->get_parameters(), $parameters['href']->get_include_session());
      }
    }

    return $parameters;
  }

/**
 * Function to return Name Based URL link with core generation for error fallback
 * @author Bobby Easland, Rainer Schmied
 * @version 1.0
 * @param array $page Base script for URL
 * @param array $parameters URL parameters
 * @param boolean $add_session_id Switch to add osCsid
 * @return string Formed href link
 */
  function href_link($page = '', $parameters = '', $add_session_id = true) {

    if ( !in_array($page, $this->attributes['SEO_PAGES']) || MODULE_NB_URLS_STATUS == 'False' ) {
      return Href::hook($page, $parameters);
    }

    $link = $this->base_url;
    $separator = '?';

    if ([] !== $parameters) {
      $link .= $this->parse_parameters($page, $parameters, $separator);
    } else {
      $link .= $page;
    }

    return $link;
  } # end function

/**
 * SFunction to parse the parameters into an SEO URL
 * @author Bobby Easland
 * @version 1.2
 * @param string $page
 * @param string $params
 * @param string $separator NOTE: passed by reference
 * @return string
 */
  function parse_parameters($page, $params, &$separator) {

    krsort($params);

    $container = [];
    foreach ($params as $param => $value){
      switch ($param){
        case 'products_id':
          switch(true){
            case ( $page == 'product_info.php'):
              $url = $this->make_url($page, $this->get_product_name($value), $param, $value, '.html');
              break;
            case ( $page == 'ext/modules/content/reviews/write.php' ):
              $url = $this->make_url($page, $this->get_product_name($value), 'products_id_review', $value, '.html');
              break;
            default:
              $container[$param] = $value;
              break;
          } # end switch
          break;
        default:
          if( isset($value) ) $container[$param] = $value;
          break;
      } # end switch
    } # end foreach $p
    $url = isset($url) ? $url : $page;
    if ( count($container) > 0 ){
      if ( $imploded_params = $this->implode_assoc($container) ){
        $url .= $separator . $this->output_string( $imploded_params );
        $separator = '&';
      }
    }

    return $url;
  } # end function
/**
 * Function to return the generated name based URL
 * @author Bobby Easland
 * @version 1.0
 * @param string $page
 * @param string $string Stripped, formed anchor
 * @param string $anchor_type Parameter type (products_id, cPath, etc.)
 * @param integer $id
 * @param string $extension Default = .html
 * @param string $separator NOTE: passed by reference -- NOTE: not used so removed
 * @return string
 */
  function make_url($page, $string, $anchor_type, $id, $extension = '.html') {
    return $string . $this->reg_anchors[$anchor_type] . $id . $extension;
  } # end function

/**
 * Function to get the product name. Use product object or database query in that order of precedent
 * @author Bobby Easland, Rainer Schmied
 * @version 1.1
 * @param integer $pID
 * @return string Stripped anchor text
 */
  function get_product_name($pID) {
    global $product;

    $result = [];
    $cName = '';
    if ( ($product instanceof Product) && ($pID == $product->get('id')) ) {
      $pName = $this->strip(!Text::is_empty($product->get('seo_title'))? $product->get('seo_title') : $product->get('name'));
    } else {
      $sqlCmd = MODULE_NB_URLS_USE_SEO_TITLES == 'True' ? 'IF(products_seo_title != "", products_seo_title, products_name) as pName' : 'products_name as pName';
      $sql = sprintf(<<<'EOSQL'
SELECT %s
  FROM products_description
  WHERE products_id = %s
  LIMIT 1
EOSQL
      , $sqlCmd, (int)$pID);

      $result = $GLOBALS['db']->query($sql)->fetch_assoc();
      $pName = $this->strip( ($result['pName'] ?? '') );
    }
    $return = (!empty($cName) ? $cName . '-'.  $pName : $pName);
    return $return;
  } # end function

/**
 * Function to check if the params contains a products_id
 * @author Bobby Easland
 * @version 1.1
 * @param string $params
 * @return boolean
 */
  function is_product_string($params) {
    if (in_array('products_id', $params)) {
      return true;
    } else {
      return false;
    }
  } # end function

/**
 * Function to strip the string of punctuation and white space and apply language specific character conversion
 * @author Bobby Easland + Rainer Schmied
 * @version 1.1
 * @param string $string
 * @return string Stripped text. Removes all non-alphanumeric characters.
 */

  function strip($string) {

    $pattern = MODULE_NB_URLS_REMOVE_ALL_SPEC_CHARS == 'True' ? "([^[:alnum:]])" : "/[^a-z0-9- ]/i";
    $string = preg_replace('/((&#39))/', '-', strtolower($string ?? '')); //remove apostrophe - not caught by above
    $anchor = preg_replace($pattern, '', strtolower($string));
    $pattern = "([[:space:]]|[[:blank:]])";
    $anchor = preg_replace($pattern, '-', $anchor);
    return $this->short_name($anchor); // return the short filtered name
  } # end function

/**
 * Function to expand the SEO_CONVERT_SET group
 * @author Bobby Easland
 * @version 1.0
 * @param string $set
 * @return mixed
 */
  function expand($set) {
    $container = [];
    if ( !empty($set) ){
      if ( $data = @explode(',', $set) ){
        foreach ( $data as $index => $valuepair){
          $p = @explode('=>', $valuepair);
          $container[trim($p[0])] = trim($p[1]);
        }
        return $container;
      } else {
        return 'false';
      }
    } else {
      return 'false';
    }
  } # end function

/**
 * Function to return the short word filtered string
 * @author Bobby Easland
 * @version 1.0
 * @param string $str
 * @param integer $limit
 * @return string Short word filtered
 */
  function short_name($str, $limit=3) {
    $container = [];
    if ( MODULE_NB_URLS_FILTER_SHORT_WORDS != 'False' ) $limit = (int)MODULE_NB_URLS_FILTER_SHORT_WORDS;
    $foo = @explode('-', $str);
    foreach($foo as $index => $value){
      switch (true){
        case ( strlen($value) <= $limit ):
          continue 2;
        default:
          $container[] = $value;
          break;
      }
    } # end foreach

    $container = ( count($container) > 1 ? implode('-', $container) : (count($container) > 0 ? $container[0] : $str ));
    return $container;
  }

/**
 * Function to implode an associative array
 * @author Bobby Easland
 * @version 1.0
 * @param array $array Associative data array
 * @param string $inner_glue
 * @param string $outer_glue
 * @return string
 */
  function implode_assoc($array, $inner_glue='=', $outer_glue='&') {
    $output = [];
    foreach( $array as $key => $item ) {
      if ( !Text::is_empty($key) && !Text::is_empty($item) ) {
        $output[] = $key . $inner_glue . $item;
      }
    } # end foreach
    return @implode($outer_glue, $output);
  }

/**
 * Function to translate a string
 * @author Bobby Easland
 * @version 1.0
 * @param string $data String to be translated
 * @param array $parse Array of tarnslation variables
 * @return string
 */
  function parse_input_field_data($data, $parse) {
    return strtr(trim($data), $parse);
  }

/**
 * Function to output a translated or sanitized string
 * @author Bobby Easland
 * @version 1.0
 * @param string $sting String to be output
 * @param mixed $translate Array of translation characters
 * @param boolean $protected Switch for htemlspecialchars processing
 * @return string
 */
  function output_string($string, $translate = false, $protected = false) {
    if ($protected == true) {
      return htmlspecialchars($string);
    } else {
      if ($translate == false) {
        return $this->parse_input_field_data($string, ['"' => '&quot;']);
      } else {
        return $this->parse_input_field_data($string, $translate);
      }
    }
  }

} # end class
